VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ClsResizer"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'Last update: 2001/12/14

Private Const ERRBASE As Long = -2147208868 'R
Private Const RESIZE As Byte = 16

Public Enum enmDirection
    NO_DIRECTION = 0
    HORIZONTAL = 1
    VERTICAL = 2
    BOTH_DIRECTIONS = 3
End Enum

Private mlngWidth As Long
Private mlngHeight As Long
Private WithEvents mfrmBase As Form
Attribute mfrmBase.VB_VarHelpID = -1
Private mctlAdapted() As Control
Private mlngControlWidth() As Long
Private mlngControlHeight() As Long
Private mbytControlDirection() As Byte

Public Sub addControl(ByRef ctl As Control, ByVal typDirection As enmDirection, Optional ByVal ysnResize As Boolean = False)
 Dim lngCount As Long
 If ctl Is Nothing Then Err.Raise ERRBASE + 2, "ClsResizer.construct", "Parameter cannot be empty."
 If typDirection = NO_DIRECTION Then Err.Raise ERRBASE + 3, "ClsResizer.construct", IIf(ysnResize, "Resize", "Move") & " action should be asked in at least one direction."
 checkControlConflicts ctl, typDirection
 lngCount = controlCount
 Set mctlAdapted(lngCount) = ctl
 If ysnResize Then
   mlngControlHeight(lngCount) = ctl.Height
   mlngControlWidth(lngCount) = ctl.Width
   mbytControlDirection(lngCount) = typDirection + RESIZE
 Else
   mlngControlHeight(lngCount) = ctl.Top
   mlngControlWidth(lngCount) = ctl.Left
   mbytControlDirection(lngCount) = typDirection
 End If
 ReDim Preserve mctlAdapted(0 To lngCount + 1)
 ReDim Preserve mlngControlWidth(0 To lngCount + 1)
 ReDim Preserve mlngControlHeight(0 To lngCount + 1)
 ReDim Preserve mbytControlDirection(0 To lngCount + 1)
End Sub

Private Sub checkControlConflicts(ByRef ctl As Control, ByVal typDirection As enmDirection)
 Dim lngCount As Long
 lngCount = controlCount - 1
 While lngCount >= 0
      If mctlAdapted(lngCount) Is ctl Then
        If (mbytControlDirection(lngCount) And typDirection) > 0 Then
          Err.Raise ERRBASE + 4, "ClsResizer.checkControlConflicts", "Control " & ctl.name & " is already added with conflicting directions."
        End If
      End If
      lngCount = lngCount - 1
 Wend
End Sub

Public Sub construct(ByRef frm As Form)
Attribute construct.VB_Description = "The constructor. Binds the form to the resizer and creates a biderectional link between the two. Break this link with the clear method."
 Static ysnHasRunBefore As Boolean
 If ysnHasRunBefore Then
   Err.Raise ERRBASE + 1, "ClsResizer.construct", "Construct method can only be used once."
 Else
   If frm Is Nothing Then Err.Raise ERRBASE + 2, "ClsResizer.construct", "Parameter cannot be empty."
   If frm.BorderStyle <> vbSizable And frm.BorderStyle <> vbSizableToolWindow Then Err.Raise ERRBASE + 5, "ClsResizer.construct", "Form is not resizable!"
   Set mfrmBase = frm
   mlngHeight = mfrmBase.Height
   mlngWidth = mfrmBase.Width
   ysnHasRunBefore = True
 End If
End Sub

Public Function controlCount() As Long
Attribute controlCount.VB_Description = "The number of controls that is moved and / or resized. A control can have two instances if it is moved in one direction and resized in the other."
 controlCount = UBound(mctlAdapted)
End Function

Private Sub Class_Initialize()
 ReDim mctlAdapted(0 To 0)
 ReDim mlngControlHeight(0 To 0)
 ReDim mlngControlWidth(0 To 0)
 ReDim mbytControlDirection(0 To 0)
End Sub

Public Sub clear()
Attribute clear.VB_Description = "Clear this side of the bidirectional link. Call this method before the destruction of the other class"
 Dim lngCount As Long
 lngCount = controlCount - 1
 While lngCount >= 0
      Set mctlAdapted(lngCount) = Nothing
      lngCount = lngCount - 1
 Wend
 Erase mctlAdapted 'release controls
 Set mfrmBase = Nothing
End Sub

Private Sub doMove(ByVal bytDirection As Byte, ByVal lngWhich As Long, ByVal lngWidthDiff As Long, ByVal lngHeightDiff As Long)
 With mctlAdapted(lngWhich)
     If (bytDirection And enmDirection.HORIZONTAL) > 0 Then
       .Left = mlngControlWidth(lngWhich) + lngWidthDiff
     End If
     If (bytDirection And enmDirection.VERTICAL) > 0 Then
       .Top = mlngControlHeight(lngWhich) + lngHeightDiff
     End If
 End With
End Sub

Private Sub doResize(ByVal bytDirection As Byte, ByVal lngWhich As Long, ByVal lngWidthDiff As Long, ByVal lngHeightDiff As Long)
 With mctlAdapted(lngWhich)
     If (bytDirection And enmDirection.HORIZONTAL) > 0 Then
       .Width = mlngControlWidth(lngWhich) + lngWidthDiff
     End If
     If (bytDirection And enmDirection.VERTICAL) > 0 Then
       .Height = mlngControlHeight(lngWhich) + lngHeightDiff
     End If
 End With
End Sub

Private Sub mfrmBase_Resize()
 Dim lngCount As Long
 Dim lngWidthDifference As Long
 Dim lngHeightDifference As Long
 Dim bytDirection As Byte
 If (mfrmBase.ScaleWidth <> 0) And (mfrmBase.ScaleHeight <> 0) Then 'not minimized
   lngHeightDifference = mfrmBase.Height - mlngHeight
   lngWidthDifference = mfrmBase.Width - mlngWidth
   resistCompression lngWidthDifference, lngHeightDifference
   lngCount = controlCount - 1
   If lngWidthDifference + lngHeightDifference > 0 Then
     While lngCount >= 0
          bytDirection = mbytControlDirection(lngCount)
          If (bytDirection And RESIZE) > 0 Then
            doResize bytDirection, lngCount, lngWidthDifference, lngHeightDifference
          Else
            doMove bytDirection, lngCount, lngWidthDifference, lngHeightDifference
          End If
          lngCount = lngCount - 1
     Wend
   End If
 End If
End Sub

Private Sub resistCompression(ByRef lngWidthDiff As Long, ByRef lngHeightDiff As Long)
 If lngHeightDiff < 0 Then
   lngHeightDiff = 0
   mfrmBase.Height = mlngHeight
 End If
 If lngWidthDiff < 0 Then
   lngWidthDiff = 0
   mfrmBase.Width = mlngWidth
 End If
End Sub
